#!/bin/bash

PACKAGE_TYPE="$1"
OUTPUT_LINUX_DIR="$2"
USE_VECTORISATION="$3"

DIR_SOFT_NAME=`basename ${PWD}`

rm -fr ${OUTPUT_LINUX_DIR}
mkdir -p ${OUTPUT_LINUX_DIR}

function createBinaryArchive(){
	EXTENTION="$1"
	echo "Create binary archive for extension '${EXTENTION}'"
	
	if [ ! -z "$1" ]
	then
		cd ..
		mv $DIR_SOFT_NAME $DIR_SOFT_NAME-${EXTENTION}
		cd $DIR_SOFT_NAME-${EXTENTION}
	fi
	
	if [ -d build ]
	then
		echo "Remove existing directory build"
		rm -fr build
	fi
	
	mkdir -p build
	cd build
	
	if [ -z "$1" ]
	then
		cmake .. -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr || cmake3 .. -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr
	else
		cmake .. -D${EXTENTION}=yes -DPROGRAM_EXTENSION=${EXTENTION} -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr || cmake3 .. -D${EXTENTION}=yes -DPROGRAM_EXTENSION=${EXTENTION} -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr
	fi
# 	make -j `nproc`
	make
	cpack
	cd ..
	if [ ! -z "$1" ]
	then
		cd ..
		mv $DIR_SOFT_NAME-${EXTENTION} $DIR_SOFT_NAME
		cd $DIR_SOFT_NAME
	fi
}

function createPackage(){
	PACKAGE_TYPE="$1"
	EXTENTION="$2"
	echo "Create Package '${PACKAGE_TYPE}' for extension '${EXTENTION}'"
	
	if [ ! -z "$1" ]
	then
		cd ..
		mv $DIR_SOFT_NAME $DIR_SOFT_NAME-${EXTENTION}
		cd $DIR_SOFT_NAME-${EXTENTION}
	fi
	
	if [ -d build ]
	then
		echo "Remove existing directory build"
		rm -fr build
	fi
	mkdir -p build
	cd build
# 	The  -DCMAKE_INSTALL_PREFIX=/usr is needed because the install prefix of CPack is /usr
	if [ -z "$2" ]
	then
		cmake .. -DCREATE_${PACKAGE_TYPE}=yes -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr || cmake3 .. -DCREATE_${PACKAGE_TYPE}=yes -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr
	else
		cmake .. -D${EXTENTION}=yes -DPROGRAM_EXTENSION=${EXTENTION} -DCREATE_${PACKAGE_TYPE}=yes -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr || cmake3 .. -D${EXTENTION}=yes -DPROGRAM_EXTENSION=${EXTENTION} -DCREATE_${PACKAGE_TYPE}=yes -DSELF_TESTS_MODE=no -DCMAKE_INSTALL_PREFIX=/usr
	fi
# 	make -j `nproc`
	make
	cpack
	cd ..
	if [ ! -z "$1" ]
	then
		cd ..
		mv $DIR_SOFT_NAME-${EXTENTION} $DIR_SOFT_NAME
		cd $DIR_SOFT_NAME
	fi
}

echo "Create package, activate extension '${USE_VECTORISATION}'"
if [[ "$USE_VECTORISATION" == "true"* ]]
then
	echo "Create packages for set of extension (AVX, AVX2, AVX512F, ...)"
	for extension in "AVX512_2" "AVX512VPOPCNTDQ" "AVX512BW" "AVX512PF" "AVX512ER" "AVX512F" "AVX2" "AVX" "SSE4.2" "SSE4.1" "SSSE3" "SSE2" "NOVECTORIZATION"
	do
		createPackage ${PACKAGE_TYPE} ${extension}
		cp build/${DIR_SOFT_NAME}-* "${OUTPUT_LINUX_DIR}"
		createBinaryArchive ${extension}
		cp build/${DIR_SOFT_NAME}-* "${OUTPUT_LINUX_DIR}"
	done
else
	echo "Create packages for generic extension"
	createPackage ${PACKAGE_TYPE}
	cp build/${DIR_SOFT_NAME}-* "${OUTPUT_LINUX_DIR}"
	createBinaryArchive
	cp build/${DIR_SOFT_NAME}-* "${OUTPUT_LINUX_DIR}"
fi




