add_custom_target(run_all)
add_custom_target(plot_all)
add_custom_target(plot_thread)

add_dependencies(plot_thread run_all)
add_dependencies(plot_all plot_thread)

set(OUTPUT_PERF_DIR "${CMAKE_BINARY_DIR}/Performances" CACHE STRING "Directory which contains output performance tests")

if(NO_PERF_REFRESH)
	message(STATUS "Activate mode to avoid performance test refreshing : NO_PERF_REFRESH = yes")
else()
	message(STATUS "Classical mode for performance test refreshing : NO_PERF_REFRESH = no")
endif()

# Run the given target
# 	targetName : name of the target to be runned
function(phoenix_runAllExample targetName)
	add_custom_command(OUTPUT ${OUTPUT_PERF_DIR}/${targetName}.txt
		COMMAND ${CMAKE_CURRENT_BINARY_DIR}/${targetName} 2> ${OUTPUT_PERF_DIR}/${targetName}.txt
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Run ${targetName} program"
		DEPENDS ${targetName}
	)
	add_custom_target("run_${targetName}"  DEPENDS ${OUTPUT_PERF_DIR}/${targetName}.txt)
	add_dependencies("run_${targetName}" ${targetName})
	add_dependencies(run_all "run_${targetName}")
endfunction(phoenix_runAllExample)

# Run the given target
# 	targetName : name of the target to be runned
function(phoenix_runExample targetName)
	set(OUTPUT_PERF_FILE ${OUTPUT_PERF_DIR}/${targetName}.txt)
	if(NO_PERF_REFRESH)
		if(NOT EXISTS ${OUTPUT_PERF_FILE})
			phoenix_runAllExample("${targetName}" "${configToRun}")
		endif()
	else()
		phoenix_runAllExample("${targetName}" "${configToRun}")
	endif()
endfunction(phoenix_runExample)

# nice --10 perl test.pl

# Run the given target
# 	targetName : name of the target to be runned
#	configToRun : configuration of the number of element of the table and the number of repetitions to be done
function(phoenix_runExampleAllConfig targetName configToRun)
	set(OUTPUT_PERF_FILE ${OUTPUT_PERF_DIR}/${targetName}.txt)
	string(REPLACE " " "" configToRun2 "${configToRun}")
	string(REPLACE "\n" "" configToRun3 "${configToRun2}")
	string(REPLACE "\t" "" configToRun4 "${configToRun3}")

	add_custom_command(OUTPUT ${OUTPUT_PERF_FILE}
# 		COMMAND nice --20 ${CMAKE_CURRENT_BINARY_DIR}/${targetName} -n "\"${configToRun4}\"" 2> ${OUTPUT_PERF_FILE}
		COMMAND ${CMAKE_CURRENT_BINARY_DIR}/${targetName} "\"${configToRun4}\"" 2> ${OUTPUT_PERF_FILE}
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Run ${targetName} program"
		DEPENDS ${targetName}
	)
	
	add_custom_target("run_${targetName}"  DEPENDS ${OUTPUT_PERF_FILE})
	add_dependencies("run_${targetName}" ${targetName})
	add_dependencies(run_all "run_${targetName}")
endfunction(phoenix_runExampleAllConfig)

# Run the given target
# 	targetName : name of the target to be runned
#	configToRun : configuration of the number of element of the table and the number of repetitions to be done
function(phoenix_runExampleConfig targetName configToRun)
	set(OUTPUT_PERF_FILE ${OUTPUT_PERF_DIR}/${targetName}.txt)
	if(NO_PERF_REFRESH)
		if(NOT EXISTS ${OUTPUT_PERF_FILE})
			phoenix_runExampleAllConfig("${targetName}" "${configToRun}")
		endif()
	else()
		phoenix_runExampleAllConfig("${targetName}" "${configToRun}")
	endif()
endfunction(phoenix_runExampleConfig)

# Run the given target
# 	targetName : name of the target to be runned
#	configToRunX : configuration of the number of element of the table and the number of repetitions to be done
#	configToRunY : configuration of the number of element of the table and the number of repetitions to be done
function(phoenix_runAllExampleConfig2d targetName configToRunX configToRunY)
	string(REPLACE " " "" configToRunX2 "${configToRunX}")
	string(REPLACE "\n" "" configToRunX3 "${configToRunX2}")
	string(REPLACE "\t" "" configToRunX4 "${configToRunX3}")
	
	string(REPLACE " " "" configToRunY2 "${configToRunY}")
	string(REPLACE "\n" "" configToRunY3 "${configToRunY2}")
	string(REPLACE "\t" "" configToRunY4 "${configToRunY3}")
	
	add_custom_command(OUTPUT ${OUTPUT_PERF_DIR}/${targetName}.txt
# 		COMMAND nice --20 ${CMAKE_CURRENT_BINARY_DIR}/${targetName} -n "\"${configToRun4}\"" 2> ${OUTPUT_PERF_DIR}/${targetName}.txt
		COMMAND ${CMAKE_CURRENT_BINARY_DIR}/${targetName} "\"${configToRunX4}\"" "\"${configToRunY4}\"" 2> ${OUTPUT_PERF_DIR}/${targetName}.txt
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Run ${targetName} program"
		DEPENDS ${targetName}
	)
	add_custom_target("run_${targetName}"  DEPENDS ${OUTPUT_PERF_DIR}/${targetName}.txt)
	add_dependencies("run_${targetName}" ${targetName})
	add_dependencies(run_all "run_${targetName}")
endfunction(phoenix_runAllExampleConfig2d)

# Run the given target
# 	targetName : name of the target to be runned
#	configToRunX : configuration of the number of element of the table and the number of repetitions to be done
#	configToRunY : configuration of the number of element of the table and the number of repetitions to be done
function(phoenix_runExampleConfig2d targetName configToRunX configToRunY)
	set(OUTPUT_PERF_FILE ${OUTPUT_PERF_DIR}/${targetName}.txt)
	if(NO_PERF_REFRESH)
		if(NOT EXISTS ${OUTPUT_PERF_FILE})
			phoenix_runAllExampleConfig2d("${targetName}" "${configToRunX}" "${configToRunY}")
		endif()
	else()
		phoenix_runAllExampleConfig2d("${targetName}" "${configToRunX}" "${configToRunY}")
	endif()
endfunction(phoenix_runExampleConfig2d)

# Compile and run the given target
# 	targetName : name of the target to be runned
# 	compilationOption : compilation option of the given program
#	configToRun : configuration of the number of element of the table and the number of repetitions to be done
# 	ARGN : source of the program to be compiled and launched
# 	An extra variable EXTRA_DEPENDENCIES can be defined to add library dependencies (with target_link_libraries) to the created executable
function(phoenix_compileAndRunExample targetName compilationOption configToRun)
	add_executable(${targetName} ${ARGN})
	set_property(TARGET ${targetName} PROPERTY COMPILE_FLAGS ${compilationOption})
	target_link_libraries(${targetName} ${EXTRA_DEPENDENCIES} micro_benchmark pthread)
	phoenix_runExampleConfig(${targetName} "${configToRun}")
endfunction(phoenix_compileAndRunExample)

# Compile and run the given target
# 	targetName : name of the target to be runned
# 	compilationOption : compilation option of the given program
#	configToRunX : configuration of the number of element of the table and the number of repetitions to be done
#	configToRunY : configuration of the number of element of the table and the number of repetitions to be done
# 	ARGN : source of the program to be compiled and launched
# 	An extra variable EXTRA_DEPENDENCIES can be defined to add library dependencies (with target_link_libraries) to the created executable
function(phoenix_compileAndRunExample2d targetName compilationOption configToRunX configToRunY)
	add_executable(${targetName} ${ARGN})
	set_property(TARGET ${targetName} PROPERTY COMPILE_FLAGS ${compilationOption})
	target_link_libraries(${targetName} ${EXTRA_DEPENDENCIES} micro_benchmark pthread)
	phoenix_runExampleConfig2d(${targetName} "${configToRunX}" "${configToRunY}")
endfunction(phoenix_compileAndRunExample2d)

# Run the given target
# 	targetName : name of the target to be runned
#	configToRun : configuration of the number of element of the table and the number of repetitions to be done
function(phoenix_runAllExampleParallelConfig targetName configToRun)
	string(REPLACE " " "" configToRun2 "${configToRun}")
	string(REPLACE "\n" "" configToRun3 "${configToRun2}")
	string(REPLACE "\t" "" configToRun4 "${configToRun3}")
	add_custom_command(OUTPUT ${OUTPUT_PERF_DIR}/${targetName}.txt
		COMMAND ${CMAKE_CURRENT_BINARY_DIR}/${targetName} -n "\"${configToRun4}\"" 2> ${OUTPUT_PERF_DIR}/${targetName}.txt
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Run ${targetName} program"
		DEPENDS ${targetName}
	)
	add_custom_target("run_${targetName}"  DEPENDS ${OUTPUT_PERF_DIR}/${targetName}.txt)
	add_dependencies("run_${targetName}" ${targetName})
	add_dependencies(run_all "run_${targetName}")
endfunction(phoenix_runAllExampleParallelConfig)

# Run the given target
# 	targetName : name of the target to be runned
#	configToRun : configuration of the number of element of the table and the number of repetitions to be done
function(phoenix_runExampleParallelConfig targetName configToRun)
	set(OUTPUT_PERF_FILE ${OUTPUT_PERF_DIR}/${targetName}.txt)
	if(NO_PERF_REFRESH)
		if(NOT EXISTS ${OUTPUT_PERF_FILE})
			phoenix_runAllExampleParallelConfig("${targetName}" "${configToRun}")
		endif()
	else()
		phoenix_runAllExampleParallelConfig("${targetName}" "${configToRun}")
	endif()
endfunction(phoenix_runExampleParallelConfig)

# Compile and run the given target
# 	targetName : name of the target to be runned
# 	compilationOption : compilation option of the given program
#	configToRun : configuration of the number of element of the table and the number of repetitions to be done
# 	ARGN : source of the program to be compiled and launched
function(phoenix_compileAndRunParallelExample targetName compilationOption configToRun)
	add_executable(${targetName} ${ARGN})
	set_property(TARGET ${targetName} PROPERTY COMPILE_FLAGS ${compilationOption})
	target_link_libraries(${targetName} ${EXTRA_DEPENDENCIES} micro_benchmark pthread)
	phoenix_runExampleParallelConfig(${targetName} "${configToRun}")
endfunction(phoenix_compileAndRunParallelExample)

# Run the given python script
# 	scriptName : name of the script to be ran
# 	installModuleDependency : dependency of the script
function(phoenix_runAllPythonExample scriptName installModuleDependency)
	get_filename_component(targetName ${scriptName} NAME_WE)
	
	add_custom_command(OUTPUT ${OUTPUT_PERF_DIR}/${targetName}.txt
		COMMAND ${PYTHON_EXECUTABLE} ${CMAKE_CURRENT_SOURCE_DIR}/${scriptName} 2> ${OUTPUT_PERF_DIR}/${targetName}.txt
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Run ${PYTHON_EXECUTABLE} ${scriptName} program with target ${targetName}"
		DEPENDS ${scriptName}
	)
	add_custom_target("run_${targetName}"  DEPENDS ${OUTPUT_PERF_DIR}/${targetName}.txt)
	add_dependencies("run_${targetName}" ${installModuleDependency})
	add_dependencies(run_all "run_${targetName}")
endfunction(phoenix_runAllPythonExample)

# Run the given python script
# 	scriptName : name of the script to be ran
# 	installModuleDependency : dependency of the script
function(phoenix_runPythonExample scriptName installModuleDependency)
	get_filename_component(targetName ${scriptName} NAME_WE)
	set(OUTPUT_PERF_FILE ${OUTPUT_PERF_DIR}/${targetName}.txt)
	if(NO_PERF_REFRESH)
		if(NOT EXISTS ${OUTPUT_PERF_FILE})
			phoenix_runAllPythonExample("${scriptName}" "${installModuleDependency}")
		endif()
	else()
		phoenix_runAllPythonExample("${scriptName}" "${installModuleDependency}")
	endif()
endfunction(phoenix_runPythonExample)


# Plot the performances of the different output
#	baseOutputPng : base name of the png output file
# 	dirToBeUsed : directory to be used for plot
# 	logScaleX : yes to put the x axis in log scale
#	timeUnit : unit of time (ns et cy)
#	ARGN : list of target to be compared
function(phoenix_plotPerfDir baseOutputPng dirToBeUsed logScaleX timeUnit)
	set(GNUPLOT_FILE "${dirToBeUsed}/${baseOutputPng}.gnuplot")
	file(WRITE ${GNUPLOT_FILE} "set terminal png notransparent crop enhanced size 800,600 font \"arial,14\"\n")
	file(APPEND ${GNUPLOT_FILE} "set grid xtics ytics mytics\n")
# 	file(APPEND ${GNUPLOT_FILE} "set key bottom right\n")
	file(APPEND ${GNUPLOT_FILE} "set key out vert center top\n")
	if(logScaleX)
		file(APPEND ${GNUPLOT_FILE} "set logscale x\n")
	endif(logScaleX)
	file(APPEND ${GNUPLOT_FILE} "set logscale y\n")
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"nb elements\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"elapsed time per element [${timeUnit}/el]\"\n")
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}ElapsedTimeCyEl.png\"\n")
	file(APPEND ${GNUPLOT_FILE} "plot ")
	
	set(listDepend)
	foreach(inputTarget ${ARGN})
		string(REPLACE "_" " " legendStr ${inputTarget})
		file(APPEND ${GNUPLOT_FILE} "\"${inputTarget}.txt\" using 1:2:(\$2-\$4):(\$2+\$4) title \"${legendStr}\" with yerrorlines lw 2,")
		list(APPEND listDepend "${dirToBeUsed}/${inputTarget}.txt")
# 		message(STATUS "add dependency ${dirToBeUsed}/${inputTarget}.txt")
	endforeach(inputTarget)
	file(APPEND ${GNUPLOT_FILE} "\n")
	
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"nb elements\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"elapsed time [${timeUnit}]\"\n")
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}ElapsedTime.png\"\n")
	file(APPEND ${GNUPLOT_FILE} "plot ")
	
	foreach(inputTarget ${ARGN})
		string(REPLACE "_" " " legendStr ${inputTarget})
		file(APPEND ${GNUPLOT_FILE} "\"${inputTarget}.txt\" using 1:3:(\$3-\$5):(\$3+\$5) title \"${legendStr}\" with yerrorlines lw 2,")
	endforeach(inputTarget)
	file(APPEND ${GNUPLOT_FILE} "\n")
	
	add_custom_command(OUTPUT ${dirToBeUsed}/${baseOutputPng}ElapsedTimeCyEl.png ${dirToBeUsed}/${baseOutputPng}ElapsedTime.png
		COMMAND gnuplot  ${GNUPLOT_FILE}
		WORKING_DIRECTORY "${dirToBeUsed}"
		COMMENT "Call gnuplot ${baseOutputPng}"
		DEPENDS ${listDepend}
	)
	add_custom_target("plot_${baseOutputPng}" DEPENDS ${dirToBeUsed}/${baseOutputPng}ElapsedTimeCyEl.png ${dirToBeUsed}/${baseOutputPng}ElapsedTime.png)
	
# 	foreach(inputTarget ${ARGN})
# 		add_dependencies("plot_${baseOutputPng}" "run_${inputTarget}")
# 	endforeach(inputTarget)
	
	
	add_dependencies(plot_all "plot_${baseOutputPng}")
endfunction(phoenix_plotPerfDir)


# Plot the performances of the different output
#	baseOutputPng : base name of the png output file
#	ARGN : list of target to be compared
function(phoenix_plotPerf baseOutputPng)
	phoenix_plotPerfDir("${baseOutputPng}" "${OUTPUT_PERF_DIR}" no "ns" ${ARGN})
endfunction(phoenix_plotPerf)

# Plot the performances of the different output but the X axis is displayed in log
#	baseOutputPng : base name of the png output file
#	ARGN : list of target to be compared
function(phoenix_plotPerfLogX baseOutputPng)
	phoenix_plotPerfDir("${baseOutputPng}" "${OUTPUT_PERF_DIR}" yes "ns" ${ARGN})
endfunction(phoenix_plotPerfLogX)

# Plot the performances of the different output
#	baseOutputPng : base name of the png output file
#	ARGN : list of target to be compared
function(phoenix_plotPerfCy baseOutputPng)
	phoenix_plotPerfDir("${baseOutputPng}" "${OUTPUT_PERF_DIR}" no "cy" ${ARGN})
endfunction(phoenix_plotPerfCy)

# Plot the performances of the different output but the X axis is displayed in log
#	baseOutputPng : base name of the png output file
#	ARGN : list of target to be compared
function(phoenix_plotPerfCyLogX baseOutputPng)
	phoenix_plotPerfDir("${baseOutputPng}" "${OUTPUT_PERF_DIR}" yes "cy" ${ARGN})
endfunction(phoenix_plotPerfCyLogX)


# Plot the performances of the different output with probability on X axis and no more nb elements
# baseOutputPng : base name of the png output file
# ARGN : list of target to be compared
function(phoenix_plotPerfProba baseOutputPng)
	set(GNUPLOT_FILE "${OUTPUT_PERF_DIR}/${baseOutputPng}.gnuplot")
	file(WRITE ${GNUPLOT_FILE} "set terminal png notransparent crop enhanced size 800,600 font \"arial,14\"\n")
	file(APPEND ${GNUPLOT_FILE} "set grid xtics ytics mytics\n")
	file(APPEND ${GNUPLOT_FILE} "set key bottom right\n")
	file(APPEND ${GNUPLOT_FILE} "set logscale y\n")
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"proba\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"elapsed time per element [cy/el]\"\n")
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}ElapsedTimeCyEl.png\"\n")
	file(APPEND ${GNUPLOT_FILE} "plot ")
	
	set(listDepend)
	foreach(inputTarget ${ARGN})
		string(REPLACE "_" " " legendStr ${inputTarget})
		file(APPEND ${GNUPLOT_FILE} "\"${inputTarget}.txt\" using 1:2 title \"${legendStr}\" with lines  lw 2,")
		list(APPEND listDepend "${OUTPUT_PERF_DIR}/${inputTarget}.txt")
	endforeach(inputTarget)
	file(APPEND ${GNUPLOT_FILE} "\n")
	
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"proba\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"elapsed time [cy]\"\n")
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}ElapsedTime.png\"\n")
	file(APPEND ${GNUPLOT_FILE} "plot ")
	
	foreach(inputTarget ${ARGN})
		string(REPLACE "_" " " legendStr ${inputTarget})
		file(APPEND ${GNUPLOT_FILE} "\"${inputTarget}.txt\" using 1:3 title \"${legendStr}\" with lines  lw 2,")
	endforeach(inputTarget)
	file(APPEND ${GNUPLOT_FILE} "\n")
	
	add_custom_command(OUTPUT ${OUTPUT_PERF_DIR}/${baseOutputPng}ElapsedTimeCyEl.png ${OUTPUT_PERF_DIR}/${baseOutputPng}ElapsedTime.png
		COMMAND gnuplot  ${GNUPLOT_FILE}
		WORKING_DIRECTORY "${OUTPUT_PERF_DIR}"
		COMMENT "Call gnuplot ${baseOutputPng}"
		DEPENDS ${listDepend}
	)
	add_custom_target("plot_${baseOutputPng}"  DEPENDS ${OUTPUT_PERF_DIR}/${baseOutputPng}ElapsedTimeCyEl.png ${OUTPUT_PERF_DIR}/${baseOutputPng}ElapsedTime.png)
# 	foreach(inputTarget ${ARGN})
# 		add_dependencies("plot_${baseOutputPng}" "run_${inputTarget}")
# 	endforeach(inputTarget)
	add_dependencies(plot_all "plot_${baseOutputPng}")
endfunction(phoenix_plotPerfProba)

# Create the split file for parallel performances
# 	nbMaxCore : maximum number of cores used in the tests
function(phoenix_splitParallelFile nbMaxCore)
	file(GLOB parallelFile "${CMAKE_CURRENT_SOURCE_DIR}/parallel*.txt")
	
	foreach(fileName ${parallelFile})
		get_filename_component(targetName ${fileName} NAME_WE)
		
		set(fullTargetName "run_${targetName}_split")

		set(listDepend)
		foreach(nbThread RANGE 1 ${nbMaxCore})
			set(targetNameThread "${targetName}_nbThr_${nbThread}")
			list(APPEND listDepend "${CMAKE_CURRENT_BINARY_DIR}/${targetNameThread}.txt")
		endforeach(nbThread)
		
		add_custom_command(OUTPUT ${listDepend}
			COMMAND ${SCRIPT_SPLIT_DATA} "${fileName}" ${nbMaxCore}
			WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
			COMMENT "Split parallel file ${fileName}"
		)
		add_custom_target(${fullTargetName} DEPENDS ${listDepend})
	endforeach(fileName)
endfunction(phoenix_splitParallelFile)


# Plot the performances of the different output
#	baseOutputPng : base name of the png output file
#	ARGN : list of target to be compared
function(phoenix_plotPerfThread baseOutputPng)
	set(GNUPLOT_FILE "${CMAKE_CURRENT_BINARY_DIR}/${baseOutputPng}.gnuplot")
	file(WRITE ${GNUPLOT_FILE} "set terminal png notransparent crop enhanced size 800,600 font \"arial,14\"\n")
	file(APPEND ${GNUPLOT_FILE} "set grid xtics ytics mytics\n")
	file(APPEND ${GNUPLOT_FILE} "set key bottom right\n")
	file(APPEND ${GNUPLOT_FILE} "set logscale x\n")
	file(APPEND ${GNUPLOT_FILE} "set logscale y\n")
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"nb elements\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"elapsed time per element [cy/el]\"\n")
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}ElapsedTimeCyEl.png\"\n")
	file(APPEND ${GNUPLOT_FILE} "plot ")
	
	set(listDepend)
	foreach(targetName ${ARGN})
		get_filename_component(inputFile ${targetName} NAME_WE)
		string(REPLACE "_" " " legendStr ${inputFile})
		file(APPEND ${GNUPLOT_FILE} "\"${targetName}\" using 1:2 title \"${legendStr}\" with lines  lw 2,")
		list(APPEND listDepend "${CMAKE_CURRENT_BINARY_DIR}/${targetName}")
	endforeach(targetName)
	file(APPEND ${GNUPLOT_FILE} "\n")
	
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"nb elements\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"elapsed time [cy]\"\n")
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}ElapsedTime.png\"\n")
	file(APPEND ${GNUPLOT_FILE} "plot ")
	
	foreach(targetName ${ARGN})
		get_filename_component(inputFile ${targetName} NAME_WE)
		string(REPLACE "_" " " legendStr ${inputFile})
		file(APPEND ${GNUPLOT_FILE} "\"${targetName}\" using 1:3 title \"${legendStr}\" with lines  lw 2,")
	endforeach(targetName)
	file(APPEND ${GNUPLOT_FILE} "\n")
	
	add_custom_command(OUTPUT ${CMAKE_CURRENT_BINARY_DIR}/${baseOutputPng}ElapsedTimeCyEl.png ${CMAKE_CURRENT_BINARY_DIR}/${baseOutputPng}ElapsedTime.png
		COMMAND gnuplot  ${GNUPLOT_FILE}
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Call gnuplot ${baseOutputPng}"
		DEPENDS ${listDepend}
	)
	add_custom_target("plot_${baseOutputPng}" DEPENDS ${CMAKE_CURRENT_BINARY_DIR}/${baseOutputPng}ElapsedTimeCyEl.png ${CMAKE_CURRENT_BINARY_DIR}/${baseOutputPng}ElapsedTime.png ${listDepend})
	
	add_dependencies(plot_thread "plot_${baseOutputPng}")
endfunction(phoenix_plotPerfThread)




# Plot the performances of the different output
#	baseOutputPng : base name of the png output file
# 	inputTarget : target to be plotted
# 	dirToBeUsed : directory to be used for plot
# 	labelX : label of the x axis
# 	labelY : label of the y axis
# 	labelCB : label of the color bar (or z axis)
# 	logScaleX : yes to put the x axis in log scale
# 	logScaleY : yes to put the y axis in log scale
# 	logScaleCB : yes to put the color bar axis in log scale (or z axis)
function(phoenix_plotPerfDir2d baseOutputPng inputTarget dirToBeUsed labelX labelY labelCB logScaleX logScaleY logScaleCB)
	set(GNUPLOT_FILE "${dirToBeUsed}/${baseOutputPng}.gnuplot")
	file(WRITE ${GNUPLOT_FILE} "set terminal png notransparent crop enhanced size 1200,800 font \"arial,14\"\n")
	file(APPEND ${GNUPLOT_FILE} "set grid xtics ytics mytics\n")
	file(APPEND ${GNUPLOT_FILE} "set lmargin at screen 0.2\n")
	file(APPEND ${GNUPLOT_FILE} "set rmargin at screen 0.7\n")
	
	file(APPEND ${GNUPLOT_FILE} "unset key\n")
	
	file(APPEND ${GNUPLOT_FILE} "set view map scale 1\n")
	file(APPEND ${GNUPLOT_FILE} "set pm3d implicit at b\n")
	
	if(logScaleX)
		file(APPEND ${GNUPLOT_FILE} "set logscale x\n")
	endif(logScaleX)
	if(logScaleY)
		file(APPEND ${GNUPLOT_FILE} "set logscale y\n")
	endif(logScaleY)
	if(logScaleCB)
		file(APPEND ${GNUPLOT_FILE} "set logscale cb\n")
	endif(logScaleCB)
	
	file(APPEND ${GNUPLOT_FILE} "set xlabel \"${labelX}\"\n")
	file(APPEND ${GNUPLOT_FILE} "set ylabel \"${labelY}\"\n")
	file(APPEND ${GNUPLOT_FILE} "set cblabel \"${labelCB}\"\n")
	
	file(APPEND ${GNUPLOT_FILE} "set output \"${baseOutputPng}.png\"\n")
	
	set(INPUT_TARGET_DATA ${dirToBeUsed}/${inputTarget}.txt)
	file(APPEND ${GNUPLOT_FILE} "splot \"${inputTarget}.txt\"\n")
	
	add_custom_command(OUTPUT ${dirToBeUsed}/${baseOutputPng}.png
		COMMAND gnuplot  ${GNUPLOT_FILE}
		WORKING_DIRECTORY "${dirToBeUsed}"
		COMMENT "Call gnuplot ${baseOutputPng}"
		DEPENDS ${INPUT_TARGET_DATA}
	)
	add_custom_target("plot_${baseOutputPng}" DEPENDS ${dirToBeUsed}/${baseOutputPng}.png)
	
# 	add_dependencies("plot_${baseOutputPng}" "run_${inputTarget}")
	
	add_dependencies(plot_all "plot_${baseOutputPng}")
endfunction(phoenix_plotPerfDir2d)

# Plot the performances of the different output
#	baseOutputPng : base name of the png output file
# 	inputTarget : target to be plotted
# 	labelX : label of the x axis
# 	labelY : label of the y axis
# 	labelCB : label of the color bar (or z axis)
# 	logScaleX : yes to put the x axis in log scale
# 	logScaleY : yes to put the y axis in log scale
# 	logScaleCB : yes to put the color bar axis in log scale (or z axis)
function(phoenix_plotPerf2d baseOutputPng inputTarget labelX labelY labelCB logScaleX logScaleY logScaleCB)
	phoenix_plotPerfDir2d("${baseOutputPng}" "${inputTarget}" "${OUTPUT_PERF_DIR}" "${labelX}" "${labelY}" "${labelCB}" ${logScaleX} ${logScaleY} ${logScaleCB})
endfunction(phoenix_plotPerf2d)


